import * as React from 'react';
import { render, unmountComponentAtNode } from 'react-dom';

import MediaInsert from '../components/media-insert';
import MediaUpdate from '../components/media-update';
import { MediaItem } from '../types/media';
import { random } from '../lib/utility';
import { remove } from '../lib/dom';

const insertMark = '.js-insert';
const editMark = '.js-edit';
const removeMark =  '.js-remove';
const previewMark =  '.js-preview';
const valueMark = '.js-value';
const dispatch = (ctx) => {
  const $target = $(valueMark, ctx);
  const $remove = $(removeMark, ctx);
  const $preview = $(previewMark, ctx);
  const $insert = $(insertMark, ctx);
  const $edit = $(editMark, ctx);
  if (!$target.val()) {
    $edit.hide();
    $remove.hide();
  }
  // メディア挿入
  $insert.off('click').on('click', (e) => {
    e.preventDefault(); 
    const id = `media-${random(10)}`;
    $('body').append(`<div id="${id}"></div>`);
    const mediaTarget = document.querySelector(`#${id}`);
    const tab = $(e.target).data('mode');
    const filetype = $(e.target).data('type') ? $(e.target).data('type') : 'all';
    render(<MediaInsert
      tab={tab}
      radioMode={true}
      filetype={filetype}
      onInsert={(items) => {
        if (!items || !items.length) {
          alert('メディアが選択されていません。');
          return;
        }
        const [ item ] = items;
        if ($target.length) {
          $target.val(item.media_id);
        }
        if ($preview.length) {
          $preview.attr('src', item.media_thumbnail);
          $preview.show();
        }
        $edit.show();
        $remove.show();
        unmountComponentAtNode(mediaTarget);
        remove(mediaTarget);
      }}
      onClose={() => {
        unmountComponentAtNode(mediaTarget);
        remove(mediaTarget);
      }}
    />, mediaTarget);
  });
  // メディア編集
  $edit.off('click').on('click', (e) => {
    e.preventDefault();
    const id = `media-${random(10)}`;
    $('body').append(`<div id="${id}"></div>`);
    const mid = $target.val();
    const mediaTarget = document.querySelector(`#${id}`);
    render(<MediaUpdate
      mid={mid}
      onClose={() => {
        unmountComponentAtNode(mediaTarget);
        remove(mediaTarget);
      }}
      onUpdate={(media: MediaItem) => {
        unmountComponentAtNode(mediaTarget);
        remove(mediaTarget);
        if ($preview.length) {
          $preview.attr('src', media.media_thumbnail);
          $target.val(media.media_id)
        }
      }}
    />, mediaTarget);
  });
  // メディア削除
  $remove.off('click').on('click', (e) => {
    e.preventDefault();
    $target.val('');
    $edit.hide();
    $remove.hide();
    $preview.attr('src', '').hide();
  });
}

export default (ctx: HTMLElement) => {
  const fields = ctx.querySelectorAll(ACMS.Config.mediaFieldMark);
  [].forEach.call(fields, (field) => {
    dispatch(field);
  })
}

